@@ -1,3 +1,4 @@
+import fnmatch
 import io
 import ntpath
 import os
@@ -14,7 +15,7 @@ from errno import EINVAL, ENOENT, EEXIST
 from functools import wraps
 from itertools import chain, count
 from operator import attrgetter
-from stat import S_ISDIR
+from stat import S_ISDIR, S_ISLNK, S_ISREG
 
 
 supports_symlinks = True
@@ -39,6 +40,12 @@ __all__ = [
 # Internals
 #
 
+def _is_wildcard_pattern(pat):
+    # Whether this pattern needs actual matching using fnmatch, or can
+    # be looked up directly as a file.
+    return "*" in pat or "?" in pat or "[" in pat
+
+
 class _Flavour(object):
     """A flavour implements a particular (platform-specific) set of path
     semantics."""
@@ -125,6 +132,9 @@ class _NTFlavour(_Flavour):
             part = part.lstrip(sep)
         return drv, root, part
 
+    def casefold(self, s):
+        return s.lower()
+
     def casefold_parts(self, parts):
         return [p.lower() for p in parts]
 
@@ -134,7 +144,7 @@ class _NTFlavour(_Flavour):
             return os.getcwd()
         if _getfinalpathname is not None:
             return self._ext_to_normal(_getfinalpathname(s))
-        # Means fallback on abspath
+        # Means fallback on absolute
         return None
 
     def _ext_to_normal(self, s):
@@ -174,6 +184,9 @@ class _PosixFlavour(_Flavour):
         else:
             return '', '', part
 
+    def casefold(self, s):
+        return s
+
     def casefold_parts(self, parts):
         return parts
 
@@ -771,6 +784,31 @@ class PurePath(object):
         by the system, if any."""
         return self._flavour.is_reserved(self._parts)
 
+    def match(self, path_pattern):
+        """
+        Return True if this path matches the given pattern.
+        """
+        cf = self._flavour.casefold
+        path_pattern = cf(path_pattern)
+        drv, root, pat_parts = self._flavour.parse_parts((path_pattern,))
+        if not pat_parts:
+            raise ValueError("empty pattern")
+        if drv and drv != cf(self._drv):
+            return False
+        if root and root != cf(self._root):
+            return False
+        parts = self._cparts
+        if drv or root:
+            if len(pat_parts) != len(parts):
+                return False
+            pat_parts = pat_parts[1:]
+        elif len(pat_parts) > len(parts):
+            return False
+        for part, pat in zip(reversed(parts), reversed(pat_parts)):
+            if not fnmatch.fnmatchcase(part, pat):
+                return False
+        return True
+
 
 class PurePosixPath(PurePath):
     _flavour = _posix_flavour
@@ -910,6 +948,39 @@ class Path(PurePath):
         # A stub for the opener argument to built-in open()
         return self._accessor.open(self, flags, mode)
 
+    def _select_children(self, pattern_parts, recursive):
+        # Helper for globbing
+        # XXX symlink loops
+        if not pattern_parts:
+            yield self
+            return
+        if not self.is_dir():
+            return
+        pat = pattern_parts[0]
+        child_parts = pattern_parts[1:]
+        if _is_wildcard_pattern(pat):
+            cf = self._flavour.casefold
+            for name in self._accessor.listdir(self):
+                name = cf(name)
+                if fnmatch.fnmatchcase(name, pat):
+                    child_path = self._make_child_relpath(name)
+                    for p in child_path._select_children(child_parts, False):
+                        yield p
+                elif recursive:
+                    child_path = self._make_child_relpath(name)
+                    for p in child_path._select_children(pattern_parts, recursive):
+                        yield p
+        else:
+            child_path = self._make_child_relpath(pat)
+            if child_path.exists():
+                for p in child_path._select_children(child_parts, False):
+                    yield p
+            if recursive:
+                for name in self._accessor.listdir(self):
+                    child_path = self._make_child_relpath(name)
+                    for p in child_path._select_children(pattern_parts, recursive):
+                        yield p
+
     # Public API
 
     @classmethod
@@ -938,7 +1009,29 @@ class Path(PurePath):
             return getattr(self._stat, name)
         return super(Path, self).__getattribute__(name)
 
-    def abspath(self):
+    def glob(self, pattern):
+        """Iterate over this subtree and yield all existing files (of any
+        kind, including directories) matching the given pattern.
+        """
+        pattern = self._flavour.casefold(pattern)
+        drv, root, pattern_parts = self._flavour.parse_parts((pattern,))
+        if drv or root:
+            raise NotImplementedError("Non-relative patterns are unsupported")
+        for p in self._select_children(pattern_parts, recursive=False):
+            yield p
+
+    def rglob(self, pattern):
+        """Recursively yield all existing files (of any kind, including
+        directories) matching the given pattern, anywhere in this subtree.
+        """
+        pattern = self._flavour.casefold(pattern)
+        drv, root, pattern_parts = self._flavour.parse_parts((pattern,))
+        if drv or root:
+            raise NotImplementedError("Non-relative patterns are unsupported")
+        for p in self._select_children(pattern_parts, recursive=True):
+            yield p
+
+    def absolute(self):
         """Return an absolute version of this path.  This function works
         even if the path doesn't point to anything.
 
@@ -969,7 +1062,7 @@ class Path(PurePath):
             # No symlink resolution => for consistency, raise an error if
             # the path doesn't exist or is forbidden
             self._stat
-            s = str(self.abspath())
+            s = str(self.absolute())
         # Now we have no symlinks in the path, it's safe to normalize it.
         normed = self._flavour.pathmod.normpath(s)
         obj = self._from_parts((normed,), init=False)
@@ -1123,6 +1216,20 @@ class Path(PurePath):
         """
         return S_ISDIR(self._stat.st_mode)
 
+    def is_file(self):
+        """
+        Whether this path is a regular file (also True for symlinks pointing
+        to regular files).
+        """
+        return S_ISREG(self._stat.st_mode)
+
+    def is_symlink(self):
+        """
+        Whether this path is a symbolic link.
+        """
+        st = self.lstat()
+        return S_ISLNK(st.st_mode)
+
 
 class PosixPath(Path, PurePosixPath):
     __slots__ = ()
